/* eslint-disable react/prop-types */
import PropTypes from 'prop-types';
import React, { useState, useEffect, useLayoutEffect, useRef } from 'react';
import cx from 'classnames';
import Menu from '@s-ui/react-icons/lib/Menu';
import DropdownBasic from '@schibstedspain/sui-dropdown-basic';
import DropdownUser from '@schibstedspain/sui-dropdown-user';
import AtomButton from '@s-ui/react-atom-button';
var DEFAULT_NAV_WRAP_STYLE = {
  top: 'inherit',
  left: 'inherit',
  height: 'inherit',
  width: 'inherit'
};
var HTML_HAS_SCROLL_DISABLED = 'html-has-scroll-disabled';
var BODY_HAS_SCROLL_DISABLED = 'body-has-scroll-disabled';
/**
 * Topbar containing a dropdown with user data (login, logout, secured links...).
 */

export default function TopbarUser(_ref) {
  var _ref$shouldDisplayNav = _ref.shouldDisplayNavUser,
      shouldDisplayNavUser = _ref$shouldDisplayNav === void 0 ? true : _ref$shouldDisplayNav,
      _ref$toggleIcon = _ref.toggleIcon,
      toggleIcon = _ref$toggleIcon === void 0 ? Menu : _ref$toggleIcon,
      brand = _ref.brand,
      navMain = _ref.navMain,
      navUser = _ref.navUser,
      navCTA = _ref.navCTA,
      _ref$linkFactory = _ref.linkFactory,
      linkFactory = _ref$linkFactory === void 0 ? function (_ref2) {
    var href = _ref2.href,
        className = _ref2.className,
        children = _ref2.children,
        target = _ref2.target,
        title = _ref2.title;
    return React.createElement("a", {
      href: href,
      className: className,
      target: target,
      title: title
    }, children);
  } : _ref$linkFactory,
      _ref$elementsToKeepSc = _ref.elementsToKeepScrollOnToggleMenu,
      elementsToKeepScrollOnToggleMenu = _ref$elementsToKeepSc === void 0 ? [] : _ref$elementsToKeepSc;

  var _topbarUserNode = useRef(null);

  var _topbarUserToggleNode = useRef(null);

  var _windowWidth = useRef();

  var _useState = useState(false),
      menuExpanded = _useState[0],
      setMenuExpanded = _useState[1];

  var _useState2 = useState(false),
      isToggleHidden = _useState2[0],
      setToggleHidden = _useState2[1];

  var _useState3 = useState(DEFAULT_NAV_WRAP_STYLE),
      navWrapStyle = _useState3[0],
      setNavWrapStyle = _useState3[1];
  /**
   * Set navigation wrap inline styles.
   */


  var _setNavWrapStyles = function _setNavWrapStyles() {
    var _topbarUserNode$curre = _topbarUserNode.current.getBoundingClientRect(),
        top = _topbarUserNode$curre.top,
        left = _topbarUserNode$curre.left,
        height = _topbarUserNode$curre.height,
        width = _topbarUserNode$curre.width;

    var navWrapTop = top + height;
    setNavWrapStyle({
      top: navWrapTop,
      left: left,
      height: window.innerHeight - navWrapTop,
      width: width
    });
  };
  /**
   * Toggle menu state: expanded/collapsed.
   */


  var _toggleMenu = function _toggleMenu() {
    setMenuExpanded(!menuExpanded);
  };
  /**
   * Handle click on navigation wrap.
   */


  var _handleNavWrapClick = function _handleNavWrapClick(_ref3) {
    var target = _ref3.target,
        currentTarget = _ref3.currentTarget;

    if (menuExpanded && target === currentTarget) {
      _toggleMenu();
    }
  };

  useLayoutEffect(function () {
    /**
     * Set the display state for toggle button.
     */
    var _setToggleDisplayState = function _setToggleDisplayState() {
      // Only go on if user has been resized the browser window horizontally.
      if (window.innerWidth === _windowWidth.current) return; // Then save the new global value again.

      _windowWidth.current = window.innerWidth;

      var _window$getComputedSt = window.getComputedStyle(_topbarUserToggleNode.current),
          display = _window$getComputedSt.display;

      var isToggleCurrentlyHidden = display === 'none';

      if (!isToggleCurrentlyHidden) {
        _setNavWrapStyles();
      }

      if (isToggleCurrentlyHidden !== isToggleHidden) {
        setToggleHidden(isToggleCurrentlyHidden);
      }
    };

    _setToggleDisplayState();

    window.addEventListener('resize', _setToggleDisplayState);
    return function () {
      window.removeEventListener('resize', _setToggleDisplayState);
    };
  }, [isToggleHidden]);
  useEffect(function () {
    var _verticalScrollPosition;
    /**
     * Lock body element scroll.
     */


    var _lockBodyScroll = function _lockBodyScroll() {
      _verticalScrollPosition = window.scrollY;
      var transformStyleToKeepScroll = "translate3d(0, -" + _verticalScrollPosition + "px, 0)";
      window.document.documentElement.classList.add(HTML_HAS_SCROLL_DISABLED);
      window.document.body.classList.add(BODY_HAS_SCROLL_DISABLED);
      elementsToKeepScrollOnToggleMenu.forEach(function (selector) {
        document.querySelector(selector).style.transform = transformStyleToKeepScroll;
      });
    };
    /**
     * Unlock body element scroll.
     */


    var _unlockBodyScroll = function _unlockBodyScroll() {
      elementsToKeepScrollOnToggleMenu.forEach(function (selector) {
        var element = document.querySelector(selector);
        if (element) element.style.transform = '';
      });
      window.document.documentElement.classList.remove(HTML_HAS_SCROLL_DISABLED);
      window.document.body.classList.remove(BODY_HAS_SCROLL_DISABLED);
      elementsToKeepScrollOnToggleMenu.length && window.scrollTo(0, _verticalScrollPosition);
    };

    if (menuExpanded && !isToggleHidden) {
      _lockBodyScroll();
    } else {
      _unlockBodyScroll();
    }
  }, [menuExpanded, isToggleHidden, elementsToKeepScrollOnToggleMenu]);
  /**
   * Render main navigation function.
   */

  var _renderNavMain = function _renderNavMain(isToggleHidden) {
    return function (_ref4, index) {
      var icon = _ref4.icon,
          text = _ref4.label,
          menu = _ref4.menu,
          arrowButtonIcon = _ref4.arrowButtonIcon;
      return React.createElement(DropdownBasic, {
        key: index,
        button: {
          icon: icon,
          text: text,
          arrowButtonIcon: arrowButtonIcon
        },
        menu: menu,
        expandOnMouseOver: isToggleHidden,
        linkFactory: linkFactory
      });
    };
  };

  var Link = linkFactory;
  var ToggleIcon = toggleIcon;
  var brandName = brand.name,
      brandUrl = brand.url;
  var avatar = navUser.avatar,
      name = navUser.name,
      menu = navUser.menu;
  var navWrapClassName = cx('sui-TopbarUser-navWrap', {
    'is-expanded': menuExpanded
  });
  var hasNotifications = navUser.menu.some(function (_ref5) {
    var notifications = _ref5.notifications;
    return Boolean(notifications);
  });
  var toggleMenuClassName = cx('sui-TopbarUser-toggle', {
    'has-notifications': hasNotifications
  });
  var navCtaUrl = navCTA.url,
      navCtaText = navCTA.text,
      _navCTA$onClick = navCTA.onClick,
      onCTAClick = _navCTA$onClick === void 0 ? function () {} : _navCTA$onClick;
  return React.createElement("div", {
    ref: _topbarUserNode,
    className: "sui-TopbarUser"
  }, React.createElement("div", {
    className: "sui-TopbarUser-wrap"
  }, React.createElement("button", {
    ref: _topbarUserToggleNode,
    className: toggleMenuClassName,
    onClick: _toggleMenu
  }, React.createElement(ToggleIcon, {
    svgClass: "sui-TopbarUser-toggleIcon"
  })), React.createElement(Link, {
    href: brandUrl,
    className: "sui-TopbarUser-brand",
    title: brandName
  }, brandName), React.createElement("div", {
    className: navWrapClassName,
    style: isToggleHidden ? DEFAULT_NAV_WRAP_STYLE : navWrapStyle,
    onClick: _handleNavWrapClick
  }, React.createElement("div", {
    className: "sui-TopbarUser-nav"
  }, React.createElement("div", {
    className: "sui-TopbarUser-navMain"
  }, navMain.map(_renderNavMain(isToggleHidden))), shouldDisplayNavUser && React.createElement("div", {
    className: "sui-TopbarUser-navUser"
  }, React.createElement(DropdownUser, {
    user: {
      avatar: avatar,
      name: name
    },
    menu: menu,
    expandOnMouseOver: true,
    hasNotifications: hasNotifications && !menuExpanded,
    linkFactory: linkFactory
  }))))), React.createElement("div", {
    className: "sui-TopbarUser-ctaButton"
  }, React.createElement(AtomButton, {
    link: true,
    href: navCtaUrl,
    title: navCtaText,
    leftIcon: React.createElement(navCTA.icon, {
      svgClass: "sui-TopbarUser-ctaButtonIcon"
    }),
    size: "small",
    type: "primary",
    onClick: onCTAClick
  }, navCtaText)));
}
TopbarUser.displayName = 'TopbarUser';